"""Ubuntu chomium webdriver"""
from selenium import webdriver

chrome_options = webdriver.ChromeOptions()
# chrome_options.add_argument('--headless')
chrome_options.add_argument('--no-sandbox')
chrome_options.add_argument('--disable-dev-shm-usage')
driver = webdriver.Chrome(executable_path='/usr/lib/chromium-browser/chromedriver', chrome_options=chrome_options)



import os

import pytest
from holviapi.tests.test_api_idempotent import test_get_invoice, test_list_invoices

from .fixtures import categoriesapi, connection, invoicesapi, productsapi

pytestmark = pytest.mark.skipif((not os.environ.get('HOLVI_POOL') or not os.environ.get('HOLVI_USER') or not os.environ.get('HOLVI_PASSWORD')), reason="HOLVI_POOL, HOLVI_USER and HOLVI_PASSWORD must be in ENV for these tests")



import os

import holvirc
import pytest
from holviapi.tests.fixtures import categoriesapi, invoicesapi, productsapi


@pytest.fixture
def connection():
    pool = os.environ.get('HOLVI_POOL', None)
    username = os.environ.get('HOLVI_USER', None)
    password = os.environ.get('HOLVI_PASSWORD', None)
    driverpath = os.environ.get('HOLVI_WEBDRIVER', 'holvirc.chromedriver.driver')
    if not pool or not username or not password:
        raise RuntimeError("HOLVI_POOL, HOLVI_USER and HOLVI_PASSWORD must be in ENV for these tests")
    cnc = holvirc.Connection.singleton(pool, username, password, driverpath)
    return cnc



"""Utility methods and classes"""
import time

import selenium.common.exceptions


# wait fors stolen from http://www.obeythetestinggoat.com/how-to-get-selenium-to-wait-for-page-load-after-a-click.html
def wait_for(condition_function, timeout=3):
    """Wait for a condition to be true"""
    start_time = time.time()
    while time.time() < start_time + timeout:
        if condition_function():
            return True
        else:
            time.sleep(0.1)
    raise RuntimeError(
        'Timeout waiting for {}'.format(condition_function.__name__)
    )


class wait_for_page_load(object):
    """Waits for page load to complete, use when submitting forms via keypresses etc"""

    def __init__(self, browser):
        self.browser = browser

    def __enter__(self):
        self.old_page = self.browser.find_element_by_tag_name('html')

    def page_has_loaded(self):
        new_page = self.browser.find_element_by_tag_name('html')
        return new_page.id != self.old_page.id

    def __exit__(self, *_):
        wait_for(self.page_has_loaded)


def element_found_by_name(driver, name):
    """Shortcut for checking that named element can be found"""
    try:
        driver.find_element_by_name(name)
        return True
    except selenium.common.exceptions.NoSuchElementException:
        return False


def dump_source_and_screen(driver, path_prefix=''):
    """Dumps a screenshot and page source"""
    driver.save_screenshot('{}screenshot.png'.format(path_prefix))
    html = driver.page_source
    with open('{}source.html'.format(path_prefix), 'wb') as fp:
        fp.write(html)



"""Remote-Control Holvi via Selenium"""
from .connection import Connection
from holviapi.errors import *
from holviapi.invoicing import Invoice, InvoiceItem, InvoiceAPI
from holviapi.checkout import Order, CheckoutAPI, CheckoutItemAnswer, CheckoutItem
from holviapi.contacts import OrderContact, InvoiceContact
from holviapi.products import ShopProduct, OrderProduct, ProductsAPI
from holviapi.categories import IncomeCategory, ExpenseCategory, CategoriesAPI



# -*- coding: utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import subprocess

import six

git_version = 'UNKNOWN'
try:
    if six.PY2:
        git_version = str(subprocess.check_output(['git', 'rev-parse', '--verify', '--short', 'HEAD'])).strip()
    if six.PY3:
        git_version = subprocess.check_output(['git', 'rev-parse', '--verify', '--short', 'HEAD']).strip().decode('ascii')
except subprocess.CalledProcessError:
    pass

setup(
    name='holvirc',
    #version='0.2.1dev-%s' % git_version,
    version='0.1.20190130',
    author='Eero "rambo" af Heurlin',
    author_email='rambo@iki.fi',
    packages=['holvirc', 'holvirc.tests'],
    license='MIT',
    long_description=open('README.md').read(),
    description='Remote-Control Holvi via Selenium',
    install_requires=list(filter(bool, (x.strip() for x in open('requirements.txt').readlines()))),
    url='https://github.com/rambo/python-holvirc',
)



import os

import pytest
from holviapi.tests.test_api_dangerous import test_create_delete_invoice, test_create_send_invoice

from .fixtures import categoriesapi, connection, invoicesapi, productsapi

pytestmark = pytest.mark.skipif((not os.environ.get('HOLVI_POOL') or not os.environ.get('HOLVI_USER') or not os.environ.get('HOLVI_PASSWORD') or not bool(os.environ.get('HOLVI_ALLOW_DANGEROUS'))), reason="HOLVI_POOL, HOLVI_USER, HOLVI_PASSWORD and HOLVI_ALLOW_DANGEROUS must be in ENV for these tests")



from __future__ import absolute_import, print_function

import functools
import importlib

import holvirc.helpers as helpers
from future.builtins import next, object
from future.utils import python_2_unicode_compatible
from holviapi.connection import Connection as HolviApiConnection
from holviapi.connection import requests
from holviapi.errors import AuthenticationError
from selenium.webdriver.common.keys import Keys

# Store multiple pool connections with singleton getter
SINGLETON_MAP = {}


@python_2_unicode_compatible
class ApiConnection(HolviApiConnection):

    @classmethod
    def singleton(self, pool, sessionid, driver_instance):
        """Get a singleton of a connection"""
        global SINGLETON_MAP
        mapkey = (pool, sessionid)
        if not mapkey in SINGLETON_MAP:
            SINGLETON_MAP[mapkey] = ApiConnection(pool, sessionid, driver_instance)
        return SINGLETON_MAP[mapkey]

    def __init__(self, poolname, sessionid, driver):
        self.pool = poolname
        self.sessionid = sessionid
        self.driver = driver

    def _init_session(self):
        """Initializes a requests.Session for us if not already initialized"""
        if not self.session:
            self.session = requests.Session()
            self.session.headers.update({
                'Content-Type': 'application/json',
            })
        self.session.remove_expired_responses()

    def sync_cookies_from_driver(self, driver=None):
        """Sync the webdriver cookies to our session"""
        if not driver:
            driver = self.driver
        self._init_session()
        for cookiedict in driver.get_cookies():
            del cookiedict['expiry'], cookiedict['httpOnly']
            self.session.cookies.set(**cookiedict)

    def sync_cookies_to_driver(self, driver=None):
        """Sync the session cookies to the webdriver"""
        if not driver:
            driver = self.driver
        self._init_session()
        for cookie in self.session.cookies:
            cookiedict = {
                'name': cookie.name,
                'value': cookie.value,
                'domain': cookie.domain,
                'path': cookie.path,
                'expiry': cookie.expires,
            }
            self.driver.add_cookie(cookiedict)

    def make_get(self, url, params={}):
        """Calls the parent method and then syncs cookies with driver"""
        ret = super(ApiConnection, self).make_get(url, params)
        self.sync_cookies_to_driver()
        return ret

    def _make_ppp(self, method, url, payload):
        """Calls the parent method and then syncs cookies with driver"""
        ret = super(ApiConnection, self)._make_ppp(method, url, payload)
        self.sync_cookies_to_driver()
        return ret


@python_2_unicode_compatible
class Connection(object):

    @classmethod
    def singleton(self, pool, username, password, driverpath='holvirc.chromedriver.driver'):
        """Get a singleton of a connection"""
        global SINGLETON_MAP
        mapkey = (pool, username, password)
        if not mapkey in SINGLETON_MAP:
            SINGLETON_MAP[mapkey] = Connection(pool, username, password, driverpath)
        return SINGLETON_MAP[mapkey]

    def __init__(self, pool, username, password, driverpath='holvirc.chromedriver.driver'):
        if isinstance(driverpath, str):
            parts = driverpath.split('.')
            attrname = parts[-1]
            modname = '.'.join(parts[:-1])
            mod = importlib.import_module(modname)
            driver = getattr(mod, attrname)
        else:
            driver = driverpath

        self.driver = driver
        self.pool = pool
        self.username = username
        self.password = password
        self.apiconnection = None
        self.login()

    @property
    def base_url_fmt(self):
        return self.apiconnection.base_url_fmt

    def login(self, username=None, password=None):
        """Log in with username and password, create API connection with the temp credentials"""
        if not username:
            username = self.username
        if not password:
            password = self.password
        self.driver.get('https://holvi.com/login/')
        helpers.wait_for(functools.partial(helpers.element_found_by_name, self.driver, "username"))
        un_input = self.driver.find_element_by_name('username')
        un_input.clear()
        un_input.send_keys(username)
        pw_input = self.driver.find_element_by_name('pass1')
        pw_input.clear()
        pw_input.send_keys(password)
        with helpers.wait_for_page_load(self.driver):
            pw_input.send_keys(Keys.RETURN)
        auth_cookie = self.driver.get_cookie('holvi_jwt_auth')
        if not auth_cookie:
            AuthenticationError("Could not find auth cookie")
        self.apiconnection = ApiConnection(self.pool, self.driver.get_cookie('sessionid')['value'], self.driver)
        self.apiconnection.sync_cookies_from_driver()

    def make_get(self, *args, **kwargs):
        """Proxy to the actual API connection handler of the same name see holviapi.Connection"""
        return self.apiconnection.make_get(*args, **kwargs)

    def make_post(self, *args, **kwargs):
        """Proxy to the actual API connection handler of the same name see holviapi.Connection"""
        return self.apiconnection.make_post(*args, **kwargs)

    def make_put(self, *args, **kwargs):
        """Proxy to the actual API connection handler of the same name see holviapi.Connection"""
        return self.apiconnection.make_put(*args, **kwargs)

    def make_patch(self, *args, **kwargs):
        """Proxy to the actual API connection handler of the same name see holviapi.Connection"""
        return self.apiconnection.make_patch(*args, **kwargs)
